package org.puremvc.as3.core.controller
{
   import org.puremvc.as3.core.view.View;
   import org.puremvc.as3.interfaces.ICommand;
   import org.puremvc.as3.interfaces.IController;
   import org.puremvc.as3.interfaces.INotification;
   import org.puremvc.as3.interfaces.IView;
   import org.puremvc.as3.patterns.observer.Observer;
   
   public class Controller implements IController
   {
      
      protected static var instance:IController;
       
      
      protected var commandMap:Array;
      
      protected var view:IView;
      
      protected const SINGLETON_MSG:String = "Controller Singleton already constructed!";
      
      public function Controller()
      {
         super();
         if(instance != null)
         {
            throw Error(SINGLETON_MSG);
         }
         instance = this;
         commandMap = new Array();
         initializeController();
      }
      
      public static function getInstance() : IController
      {
         if(instance == null)
         {
            instance = new Controller();
         }
         return instance;
      }
      
      public function removeCommand(notificationName:String) : void
      {
         commandMap[notificationName] = null;
      }
      
      public function registerCommand(notificationName:String, commandClassRef:Class) : void
      {
         if(commandMap[notificationName] == null)
         {
            view.registerObserver(notificationName,new Observer(executeCommand,this));
         }
         commandMap[notificationName] = commandClassRef;
      }
      
      protected function initializeController() : void
      {
         view = View.getInstance();
      }
      
      public function hasCommand(notificationName:String) : Boolean
      {
         return commandMap[notificationName] != null;
      }
      
      public function executeCommand(note:INotification) : void
      {
         var commandClassRef:Class = commandMap[note.getName()];
         if(commandClassRef == null)
         {
            return;
         }
         var commandInstance:ICommand = new commandClassRef();
         commandInstance.execute(note);
      }
   }
}
